// Remember to change the stm32f10x_conf.h to include the peripherals you need
/* Exported types ------------------------------------------------------------*/

/* Includes ------------------------------------------------------------------*/
#include <efs.h>
#include <ls.h>
#include <stdio.h>
#include <string.h>
#include "stm32f10x.h"
#include "SSD2119.h"
#include "SSD2119_api.h"
#include "SSD2119_api_sd.h"
#include "ads7843drv.h"
#include "SSD2119_touch.h"
#include "MicrochipGraphics.h"
#include "MicrochipButton.h"
#include "BootLogo.h"
#include "VS1053.h"

#define RED  	0xf800
#define GREEN	0x07e0
#define BLUE 	0x001f
#define WHITE	0xffff
#define BLACK	0x0000
#define YELLOW   0xFFE0

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/

// RTC Definitions
//#define RTCClockOutput_Enable  /* RTC Clock/64 is output on tamper pin(PC.13) */
#define RTC_Default_Hour 0
#define RTC_Default_Min 0
#define RTC_Default_Sec 0

void RTC_Configuration(void);
void RTC_Clock_Setup(void);

__IO u8 TimeDisplay = 0;
void Time_Adjust(uint32_t seconds);
void Time_Display(u16 x, u16 y, u16 frontColor, u16 backColor, uint32_t TimeVar);
// RTC Definitions End

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
ErrorStatus HSEStartUpStatus;

/* Private function prototypes -----------------------------------------------*/
void RCC_Configuration(void);
void GPIO_Configuration(void);
void NVIC_Configuration(void);

static __IO uint32_t TimingDelay;
void Delay(__IO uint32_t nTime);

#ifdef __GNUC__
/* With GCC/RAISONANCE, small printf (option LD Linker->Libraries->Small printf
   set to 'Yes') calls __io_putchar() */
  #define PUTCHAR_PROTOTYPE int __io_putchar(int ch)
#else
  #define PUTCHAR_PROTOTYPE int fputc(int ch, FILE *f)
#endif /* __GNUC__ */


// SD card variables
EmbeddedFileSystem efs;
DirList list;
EmbeddedFile file_r;
File file;

int TouchX, TouchY;

unsigned char  buffer[40];

/* Private function prototypes -----------------------------------------------*/

  
/* Private functions ---------------------------------------------------------*/

// Changeable settings
#define SETTINGSSAVED 0x1234
#define SETTINGSSAVED_ADDR 0x7F800
#define TOUCHABLEMENUITEMS_ADDR 0x7F802
bool TouchableMenuItems;
int Old_TouchableMenuItems;
#define SINGLECLICKMENU_ADDR 0x7F804
bool SingleClickMenu;

const float Version = 0.30;  // This isn't volatile, which means it can't be found as normal text in final hex file
volatile const u8 Serial[8] = "********"; // This is here to make it easier find, search after 2A2A2A2A2A2A2A2A in hex file
volatile const u8 FactoryDate[7] = "+++++++"; // The representation in HEX is: 1st byte=date, 2nd byte=month, 3rd byte=year msb, 4th byte=year lsb, 5th byte=hour, 6th byte=minutes, 7th byte=seconds
                                              // To find it in the HEX file, search after 2B2B2B2B2B2B2B


//#include "MainMenu.c" // The code for the Main Menu
// Application Includements
//#include "applications/SpeedTest.c"
//#include "applications/SetClock.c"
//#include "applications/Settings.c"
//#include "applications/About.c"
//#include "applications/SDCard.c"
// Application Includements end





/*******************************************************************************
* Function Name  : main
* Description    : Main program
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
int main(void)
{
  /* System clocks configuration ---------------------------------------------*/
  RCC_Configuration();

  /* NVIC configuration ------------------------------------------------------*/
  NVIC_Configuration();

  /* GPIO configuration ------------------------------------------------------*/
  GPIO_Configuration(); 

  /* Setup SysTick Timer for 1 msec interrupts  */               
  if (SysTick_Config(SystemFrequency / 1000))
  { 
    /* Capture error */ 
    while (1);
  }
 
  Lcd_Configuration();
  Lcd_Initializtion();
  
  Lcd_Clear(BLACK);
  DispPic(0,60,BootLogo); 
  RTC_Clock_Setup();  // RTC Clock setup is placed here because it takes about 1-2 seconds if it isn't setup already

  // Load settings from FLASH if they are there
  if ((*(__IO uint16_t*)(SETTINGSSAVED_ADDR)) != SETTINGSSAVED) {  
    TouchableMenuItems = 1;
    SingleClickMenu = 0;   
  } else {
    TouchableMenuItems = ((*(__IO uint16_t*)(TOUCHABLEMENUITEMS_ADDR)) & 1);
    SingleClickMenu = ((*(__IO uint16_t*)(SINGLECLICKMENU_ADDR)) & 1); 
  }

  TP_Init();
  Touch_Calibrate(0);  // Calibrate Touch Screen

while(1) 
  { MainMenu(); }
}
  

void SaveSettingsToFlash(void)
{
  /* Unlock the Flash */
  FLASH_Unlock();
  FLASH_ErasePage(SETTINGSSAVED_ADDR);

  // Touch Settings Save
  uint16_t *ptr; // Create a pointer
  ptr = &matrix; // Set the pointer to the adress of the MATRIX variable

 
  int i;
  for (i = 0; i < sizeof(MATRIX)/2; i++) {
    FLASH_ProgramHalfWord(MATRIX_FLASH_ADDR+(i*2), *ptr);
    ptr++;
  }

  if (TouchScreenIScalibrated) {
    FLASH_ProgramHalfWord(MATRIX_OPTION_FLASH_ADDR, MATRIX_SAVED_TO_FLASH);  // Set the calibration flag = Touch Screen is calibrated
  } else {
    FLASH_ProgramHalfWord(MATRIX_OPTION_FLASH_ADDR, 0xFFFF);  // Clear the calibration flag = Touch Screen is not calibrated
  }


  // User-interface Settings Save
  FLASH_ProgramHalfWord(TOUCHABLEMENUITEMS_ADDR, 0x0000 + TouchableMenuItems);
  FLASH_ProgramHalfWord(SINGLECLICKMENU_ADDR, 0x0000 + SingleClickMenu);  
  FLASH_ProgramHalfWord(SETTINGSSAVED_ADDR, SETTINGSSAVED); 

  FLASH_Lock();
}

/*******************************************************************************
* Function Name  : RCC_Configuration
* Description    : Configures the different system clocks.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void RCC_Configuration(void)
   {
   /* RCC system reset(for debug purpose) */
   RCC_DeInit();

   /* Enable HSE */
   RCC_HSEConfig(RCC_HSE_ON);

   /* Wait till HSE is ready */
   HSEStartUpStatus = RCC_WaitForHSEStartUp();

   if(HSEStartUpStatus == SUCCESS)
      {
      /* Enable Prefetch Buffer */
      FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);

      /* Flash 2 wait state */
      FLASH_SetLatency(FLASH_Latency_2);

      /* HCLK = SYSCLK */
      RCC_HCLKConfig(RCC_SYSCLK_Div1); 

      /* PCLK2 = HCLK */
      RCC_PCLK2Config(RCC_HCLK_Div1); 

      /* PCLK1 = HCLK/2 */
      RCC_PCLK1Config(RCC_HCLK_Div2);

      /* PLLCLK = 8MHz * 9 = 72 MHz */
      RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9);

      /* Enable PLL */ 
      RCC_PLLCmd(ENABLE);

      /* Wait till PLL is ready */
      while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET)
         {;}

      /* Select PLL as system clock source */
      RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

      /* Wait till PLL is used as system clock source */
      while(RCC_GetSYSCLKSource() != 0x08)
         {;}
      }

   /* Enable peripheral clocks --------------------------------------------------*/
   /* Enable GPIOA, GPIOB, GPIOC, GPIOD, GPIOE, GPIOF, GPIOG and AFIO clocks */
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB |RCC_APB2Periph_GPIOC 
         | RCC_APB2Periph_GPIOD | RCC_APB2Periph_GPIOE | RCC_APB2Periph_GPIOF | RCC_APB2Periph_GPIOG 
         | RCC_APB2Periph_AFIO, ENABLE);
   }


/*******************************************************************************
* Function Name  : GPIO_Configuration
* Description    : Configures the different GPIO ports.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void GPIO_Configuration(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;

  // GPIO Speed valid for all further GPIO configurations
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

}


/*******************************************************************************
* Function Name  : NVIC_Configuration
* Description    : Configures Vector Table base location.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void NVIC_Configuration(void)
{
#ifdef  VECT_TAB_RAM  
  /* Set the Vector Table base location at 0x20000000 */ 
  NVIC_SetVectorTable(NVIC_VectTab_RAM, 0x0); 
#else  /* VECT_TAB_FLASH  */
  /* Set the Vector Table base location at 0x08000000 */ 
  NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);   
#endif

  NVIC_InitTypeDef NVIC_InitStructure;

  /* Configure one bit for preemption priority */
  NVIC_PriorityGroupConfig(NVIC_PriorityGroup_1);

  /* Enable the RTC Interrupt */
  NVIC_InitStructure.NVIC_IRQChannel = RTC_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);
}

/**
  * @brief  Configures the RTC.
  * @param  None
  * @retval None
  */
void RTC_Configuration(void)
{
  /* Enable PWR and BKP clocks */
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR | RCC_APB1Periph_BKP, ENABLE);

  /* Allow access to BKP Domain */
  PWR_BackupAccessCmd(ENABLE);

  /* Reset Backup Domain */
  BKP_DeInit();

  /* Enable LSE */
  RCC_LSEConfig(RCC_LSE_ON);
  /* Wait till LSE is ready */
  while (RCC_GetFlagStatus(RCC_FLAG_LSERDY) == RESET)
  {}

  /* Select LSE as RTC Clock Source */
  RCC_RTCCLKConfig(RCC_RTCCLKSource_LSE);

  /* Enable RTC Clock */
  RCC_RTCCLKCmd(ENABLE);

  /* Wait for RTC registers synchronization */
  RTC_WaitForSynchro();

  /* Wait until last write operation on RTC registers has finished */
  RTC_WaitForLastTask();

  /* Enable the RTC Second */
  RTC_ITConfig(RTC_IT_SEC, ENABLE);

  /* Wait until last write operation on RTC registers has finished */
  RTC_WaitForLastTask();

  /* Set RTC prescaler: set RTC period to 1sec */
  RTC_SetPrescaler(32767); /* RTC period = RTCCLK/RTC_PR = (32.768 KHz)/(32767+1) */

  /* Wait until last write operation on RTC registers has finished */
  RTC_WaitForLastTask();
}

void RTC_Clock_Setup(void)
{
  if (BKP_ReadBackupRegister(BKP_DR1) != 0xA5A5)
  {
    /* Backup data register value is not correct or not yet programmed (when
       the first time the program is executed) */

    //printf("\r\n\n RTC not yet configured....");
    //Lcd_Text(0,0,"RTC not yet configured....",26,WHITE,BLACK);
  
    //printf("\r\n RTC configured....");
    //Lcd_Text(0,20,"RTC configured....",18,WHITE,BLACK);

    /* Adjust time by values entred by the user on the hyperterminal */
    Time_Adjust((RTC_Default_Hour*3600 + RTC_Default_Min*60 + RTC_Default_Sec));    
  }
  else
  {
    /* Check if the Power On Reset flag is set */
    if (RCC_GetFlagStatus(RCC_FLAG_PORRST) != RESET)
    {
      //printf("\r\n\n Power On Reset occurred....");
      //Lcd_Text(0,40,"Power On Reset occurred....",27,WHITE,BLACK);
    }
    /* Check if the Pin Reset flag is set */
    else if (RCC_GetFlagStatus(RCC_FLAG_PINRST) != RESET)
    {
      //printf("\r\n\n External Reset occurred....");
      //Lcd_Text(0,60,"External Reset occurred....",27,WHITE,BLACK);
    }

    //printf("\r\n No need to configure RTC....");
    //Lcd_Text(0,80,"No need to configure RTC....",28,WHITE,BLACK);
    /* Wait for RTC registers synchronization */
    RTC_WaitForSynchro();

    /* Enable the RTC Second */
    RTC_ITConfig(RTC_IT_SEC, ENABLE);
    /* Wait until last write operation on RTC registers has finished */
    RTC_WaitForLastTask();

    Delay(1000);
  }

#ifdef RTCClockOutput_Enable
  /* Enable PWR and BKP clocks */
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR | RCC_APB1Periph_BKP, ENABLE);

  /* Allow access to BKP Domain */
  PWR_BackupAccessCmd(ENABLE);

  /* Disable the Tamper Pin */
  BKP_TamperPinCmd(DISABLE); /* To output RTCCLK/64 on Tamper pin, the tamper
                                 functionality must be disabled */

  /* Enable RTC Clock Output on Tamper Pin */
  BKP_RTCOutputConfig(BKP_RTCOutputSource_CalibClock);
#endif

  /* Clear reset flags */
  RCC_ClearFlag();
}

/**
  * @brief  Adjusts time.
  * @param  None
  * @retval None
  */
void Time_Adjust(uint32_t seconds)
{
  seconds = seconds + 1; // Take care of the one second used in the RTC Setup process

  /* RTC Configuration */
  RTC_Configuration();
  
  /* Wait until last write operation on RTC registers has finished */
  RTC_WaitForLastTask();
  /* Change the current time */
  RTC_SetCounter(seconds);
  /* Wait until last write operation on RTC registers has finished */
  RTC_WaitForLastTask();

  BKP_WriteBackupRegister(BKP_DR1, 0xA5A5);
}

/**
  * @brief  Displays the current time.
  * @param  TimeVar: RTC counter value.
  * @retval None
  */
void Time_Display(u16 x, u16 y, u16 frontColor, u16 backColor, uint32_t TimeVar)
{
  char buffer[20];
  uint32_t THH = 0, TMM = 0, TSS = 0;

  /* Compute  hours */
  THH = TimeVar / 3600;
  /* Compute minutes */
  TMM = (TimeVar % 3600) / 60;
  /* Compute seconds */
  TSS = (TimeVar % 3600) % 60;

  sprintf(buffer, "Time: %0.2d:%0.2d:%0.2d", THH, TMM, TSS);
  Lcd_Text(x,y,buffer,14,frontColor,backColor);
}




void Delay(__IO uint32_t nTime)
{ 
  TimingDelay = nTime;

  while(TimingDelay != 0);
}

/**
  * @brief  Decrements the TimingDelay variable.
  * @param  None
  * @retval None
  */
void TimingDelay_Decrement(void)
{
  if (TimingDelay != 0x00)
  { 
    TimingDelay--;
  }
}





#ifdef  DEBUG
/*******************************************************************************
* Function Name  : assert_failed
* Description    : Reports the name of the source file and the source line number
*                  where the assert error has occurred.
* Input          : - file: pointer to the source file name
*                  - line: assert error line source number
* Output         : None
* Return         : None
*******************************************************************************/
void assert_failed(u8* file, u32 line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
  }
}
#endif
// Menu variables
u8 CurrentSelected, OldSelected = 0;
u8 MenuPage = 0;
bool HoldForRecalibrate = 0;
unsigned char menuTitle[25];
u8 menuTitle_length;
unsigned char menuItems[10][31];
u8 menuItems_length[10];

bool BreakNow = 0;

void Standby(void)
{
  bool BreakNow = 0;
  Lcd_Clear(BLACK);

  while (!BreakNow)
  {
    if (GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_0) == 0)
    {
      TimingDelay = 1000; // Set ms counter to 1000 to check if the hold is 1 second long

      while (GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_0) == 0)
      {
        if (TimingDelay == 0) { BreakNow = 1; break; } // Leave the standby routine
      }
      TimingDelay = 0;
    }
  }
}


void GlobalMenuInterrupt(void)
{
    /* If 1s has paased */
    if (TimeDisplay == 1)
    {
      /* Display current time */
      Time_Display(200,3,WHITE,RED,RTC_GetCounter());
    }

    if (OldSelected != CurrentSelected)
    {
      Lcd_FastRectangle(10,20*OldSelected+30,268,20*OldSelected+30+20, BLACK, 1);
      Lcd_Text(10+4, 20*OldSelected+30+3, &menuItems[OldSelected], menuItems_length[OldSelected], WHITE, BLACK);
      Lcd_FastRectangle(10,20*CurrentSelected+30,268,20*CurrentSelected+30+20, WHITE, 1);
      Lcd_Text(10+4, 20*CurrentSelected+30+3, &menuItems[CurrentSelected], menuItems_length[CurrentSelected], BLACK, WHITE);
      OldSelected = CurrentSelected;
    }
}

void ChangeMenuTitle(unsigned char *NewmenuTitle, u8 NewmenuTitle_length)
{
  u8 i;
  for (i = 0; i < NewmenuTitle_length; i++)
  {
    menuTitle[i] = *NewmenuTitle++;
  }
  menuTitle_length = NewmenuTitle_length;

  DrawMenuTitle();
}

void ChangeMenuItem(u8 itemID, unsigned char *menuItem, u8 menuItem_length)
{
  u8 i;
  for (i = 0; i < menuItem_length; i++)
  {
    menuItems[itemID][i] = *menuItem++;
  }
  menuItems_length[itemID] = menuItem_length;
}

void DrawMenuTitle(void)
{
  Lcd_FastRectangle(0,0,200,20,RED,1);
  Lcd_Text(4,3,&menuTitle, menuTitle_length, WHITE,RED);
}

void DrawMenuItems(void)
{
  u8 i;

  for (i = 0; i < 10; i++)
  {
    Lcd_FastRectangle(10,20*i+30,268,20*i+30+20, BLACK, 1);
    Lcd_Text(10+4, 20*i+30+3, &menuItems[i], menuItems_length[i], WHITE, BLACK);
  }

  Lcd_FastRectangle(10,20*CurrentSelected+30,268,20*CurrentSelected+30+20, WHITE, 1);
  Lcd_Text(10+4, 20*CurrentSelected+30+3, &menuItems[CurrentSelected], menuItems_length[CurrentSelected], BLACK, WHITE);
}

void Check_MainItems_Touch(u16 TouchX, u16 TouchY)
{
  u8 i;
  bool BreakNow = 0;
  for (i = 0; i < 10; i++) {
    if (TouchX >= 10 && TouchX <= 275 && TouchY >= 20*i+30 && TouchY <= 20*i+30+20) {
      Touch_screenToDisplay();
      TouchX = displayPoint.x;
      TouchY = displayPoint.y;
      if (!(TouchX >= 10 && TouchX <= 275 && TouchY >= 20*i+30 && TouchY <= 20*i+30+20)) { BreakNow = 1; }

      while (GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_0) == 0) {
        GlobalMenuInterrupt();  // Loop while touch screen is touched
        if (TimingDelay == 0 && HoldForRecalibrate) {
          TimingDelay = 3000;
          Lcd_Clear(BLACK);
          Lcd_Text(20,110, "Hold for 3 seconds to Recalibrate",33,WHITE,BLACK);
          while (GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_0) == 0)
          {
            if (TimingDelay == 0) {
              Touch_ReCalibrate();  // Disable Touch Screen
              BreakNow = 1;
              break;
            }
          }
          Draw_MainMenu();
          TimingDelay = 0;
          HoldForRecalibrate = 0;
          BreakNow = 1;
        }
      }

      if (BreakNow) { break; }

      if (CurrentSelected == i || SingleClickMenu)
      {
        CurrentSelected = i;
        DoAction_MainMenu(); // Menu item was already selected, so execute!
        break;
      } else {
        CurrentSelected = i;
        break;
      }
    }
  }
}

void Draw_MainMenu(void)
{
  Lcd_Clear(BLACK);
  Lcd_FastRectangle(0,0,319,20,RED,1);
  DrawMenuTitle();

  Rectangle(3, 25, 275, 235, DASHED_LINE, THICK_LINE, WHITE);

  MenuPage = 0;
  ChangeMenuTitle("Menu 1/2", 8);
  ChangeMenuItem(0, "About", 5);
  ChangeMenuItem(1, "Run Speed Test", 14);
  ChangeMenuItem(2, "SD Card", 7);
  ChangeMenuItem(3, "Menu 4", 6);
  ChangeMenuItem(4, "Menu 5", 6);
  ChangeMenuItem(5, "Standby", 7);
  ChangeMenuItem(6, "Set Clock", 9);
  ChangeMenuItem(7, "Settings", 8);
  ChangeMenuItem(8, "Recalibrate Touch Screen", 24);
  ChangeMenuItem(9, "Next Page", 9);
  DrawMenuItems();

  Lcd_FastRectangle(280, 20, 319, 239, RED, 1);
  CreateButton(0, 285, 30, 310, 50, 10, 0, "", 0); // Up button
  CreateButton(1, 285, 200, 310, 220, 10, 0, "", 0); // Down button
  CreateButton(2, 285, 110, 310, 133, 0, 0, "OK", 2); // OK button
}

void MainMenu(void)
{

  Draw_MainMenu();

   while (1)
   {
     TimingDelay = 0;
     HoldForRecalibrate = 0;
     GlobalMenuInterrupt();

     while (GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_0) == 0)
     {
       GlobalMenuInterrupt();

       // Touch Screen calibration security - if touch screen isn't calibrated probably, hold screen to recalibrate!
       if (!HoldForRecalibrate) {
         TimingDelay = 5000; // Set this to check if hold for 3 seconds
         HoldForRecalibrate = 1;
       }

       if (TimingDelay == 0 && HoldForRecalibrate) {
         TimingDelay = 3000;
         Lcd_Clear(BLACK);
         Lcd_Text(20,110, "Hold for 3 seconds to Recalibrate",33,WHITE,BLACK);
         while (GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_0) == 0)
         {
           if (TimingDelay == 0) {
             Touch_ReCalibrate();  // Disable Touch Screen
             break;
           }
         }
         Draw_MainMenu();
         TimingDelay = 0;
         HoldForRecalibrate = 0;
       }
       // Touch Screen calibration security end

       Touch_screenToDisplay();
       TouchX = displayPoint.x;
       TouchY = displayPoint.y;

       if (TouchableMenuItems) { Check_MainItems_Touch(TouchX, TouchY); }

       if (ButtonPressed(0, TouchX, TouchY)) {
         DrawButton(0, BTN_PRESSED, "", 0);
         while (GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_0) == 0)
         {
           GlobalMenuInterrupt();
         }
         if (CurrentSelected > 0) {CurrentSelected--;} else { CurrentSelected = 9; }
         DrawButton(0, 0, "", 0);
       }

       if (ButtonPressed(1, TouchX, TouchY)) {
         DrawButton(1, BTN_PRESSED, "", 0);
         while (GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_0) == 0)
         {
           GlobalMenuInterrupt();
         }
         if (CurrentSelected < 9) {CurrentSelected++;} else { CurrentSelected = 0; }
         DrawButton(1, 0, "", 0);
       }

       if (ButtonPressed(2, TouchX, TouchY)) {
         DrawButton(2, BTN_PRESSED, "OK", 2);
         while (GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_0) == 0)
         {
           GlobalMenuInterrupt();
         }
         DrawButton(2, 0, "OK", 2);  // Draw the normal (un-pressed) button
         DoAction_MainMenu();
       }
     }
   }
}


void DoAction_MainMenu(void)
{
         switch (MenuPage) {
         case 0:
           if (CurrentSelected == 0) {
             About();  // Start About application
             MenuPage = 0;
             ChangeMenuTitle("Menu 1/2", 8);
             ChangeMenuItem(0, "About", 5);
             ChangeMenuItem(1, "Run Speed Test", 14);
             ChangeMenuItem(2, "SD Card", 7);
             ChangeMenuItem(3, "Menu 4", 6);
             ChangeMenuItem(4, "Menu 5", 6);
             ChangeMenuItem(5, "Standby", 7);
             ChangeMenuItem(6, "Set Clock", 9);
             ChangeMenuItem(7, "Settings", 8);
             ChangeMenuItem(8, "Recalibrate Touch Screen", 24);
             ChangeMenuItem(9, "Next Page", 9);
             DrawMenuItems();
           }

           if (CurrentSelected == 1) {
             SpeedTest();  // Start Speed Test application
             MainMenu();
           }

           if (CurrentSelected == 2) {
             SDCard();  // Start SD Card application
             MainMenu();
           }

           if (CurrentSelected == 5) {
             Standby();  // Standby
             MainMenu();
           }

           if (CurrentSelected == 6) {
             SetClock();  // Start Set Clock application
             MainMenu();
           }

           if (CurrentSelected == 7) {
             Settings();  // Start Settings application
             MainMenu();
           }

           if (CurrentSelected == 8) {
             Touch_ReCalibrate();  // ReCalibrate Touch Screen
             MainMenu();
           }

           if (CurrentSelected == 9) {
             ChangeMenuTitle("Menu 2/2", 8);
             ChangeMenuItem(0, "Menu 11", 7);
             ChangeMenuItem(1, "Menu 12", 7);
             ChangeMenuItem(2, "Menu 13", 7);
             ChangeMenuItem(3, "Menu 14", 7);
             ChangeMenuItem(4, "Menu 15", 7);
             ChangeMenuItem(5, "Menu 16", 7);
             ChangeMenuItem(6, "Menu 17", 7);
             ChangeMenuItem(7, "Menu 18", 7);
             ChangeMenuItem(8, "Menu 19", 7);
             ChangeMenuItem(9, "Previous Page", 13);
             DrawMenuItems();
             MenuPage++;
             break;
           }

         case 1:
           if (CurrentSelected == 9) {
             ChangeMenuTitle("Menu 1/2", 8);
             ChangeMenuItem(0, "About", 5);
             ChangeMenuItem(1, "Run Speed Test", 14);
             ChangeMenuItem(2, "SD Card", 7);
             ChangeMenuItem(3, "Menu 4", 6);
             ChangeMenuItem(4, "Menu 5", 6);
             ChangeMenuItem(5, "Standby", 7);
             ChangeMenuItem(6, "Set Clock", 9);
             ChangeMenuItem(7, "Settings", 8);
             ChangeMenuItem(8, "Recalibrate Touch Screen", 24);
             ChangeMenuItem(9, "Next Page", 9);
             DrawMenuItems();
             MenuPage--;
             break;
           }
         }
}

void SpeedTest(void)
{
  char buffer[10];
  int i;

  // Line Test
  Lcd_Clear(BLACK);
  Lcd_FastRectangle(0,220,319,239,RED,1);

  i = 0;
  TimingDelay = 10;
  while(TimingDelay > 0) {
    Lcd_Line(i+2,i+4,i+6,i+10, RED);
    i++;
  }
  sprintf(buffer, "Lcd_Line (10ms): %d", i);
  Lcd_Text(8,222,buffer,17+3,WHITE,RED);
  Delay(5000);
  Lcd_Clear(BLACK);
  Lcd_FastRectangle(0,220,319,239,RED,1);

  i = 0;
  TimingDelay = 10;
  while(TimingDelay > 0) {
    Line(i+2,i+4,i+6,i+10, SOLID_LINE, NORMAL_LINE, RED);
    i++;
  }
  sprintf(buffer, "Line (10ms): %d", i);
  Lcd_Text(8,222,buffer,13+3,WHITE,RED);
  Delay(5000);
  // End of Line Test

  // Circle Test
  Lcd_Clear(BLACK);
  Lcd_FastRectangle(0,220,319,239,RED,1);

  i = 0;
  TimingDelay = 100;
  while(TimingDelay > 0) {
    Lcd_Circle(i+20,i+20, 10+i, RED, 0);
    i++;
  }
  sprintf(buffer, "Non-Filled Lcd_Circle (100ms): %d", i);
  Lcd_Text(8,222,buffer,31+2,WHITE,RED);
  Delay(5000);
  Lcd_Clear(BLACK);
  Lcd_FastRectangle(0,220,319,239,RED,1);

  i = 0;
  TimingDelay = 100;
  while(TimingDelay > 0) {
    Circle(i+20,i+20, 10+i, SOLID_LINE, NORMAL_LINE, RED);
    i++;
  }
  sprintf(buffer, "Circle(100ms): %d", i);
  Lcd_Text(8,222,buffer,15+2,WHITE,RED);
  Delay(5000);
  // End of Circle Test

  // Circle Speed Test
  Lcd_Clear(BLACK);
  Lcd_FastRectangle(0,220,319,239,RED,1);

  i = 0;
  TimingDelay = 1000;
  while(TimingDelay > 0) {
    Lcd_Circle(i+20,i+20, 10+i, RED, 1);
    i++;
  }
  sprintf(buffer, "Filled Lcd_Circle (1000ms): %d", i);
  Lcd_Text(8,222,buffer,28+2,WHITE,RED);
  Delay(5000);
  Lcd_Clear(BLACK);
  Lcd_FastRectangle(0,220,319,239,RED,1);

  i = 0;
  TimingDelay = 1000;
  while(TimingDelay > 0) {
    FillCircle(i+20,i+20, 10+i, RED);
    i++;
  }
  sprintf(buffer, "FillCircle (1000ms): %d", i);
  Lcd_Text(8,222,buffer,21+2,WHITE,RED);
  Delay(5000);
  // End of Circle Test

  // Rectangle Test
  Lcd_Clear(BLACK);
  Lcd_FastRectangle(0,220,319,239,RED,1);

  i = 0;
  TimingDelay = 100;
  while(TimingDelay > 0) {
    Lcd_FastRectangle(i+20,i+10,i+50,i+30, RED, 0);
    i++;
  }
  sprintf(buffer, "Lcd_FastRectangle (100ms): %d", i);
  Lcd_Text(8,222,buffer,27+2,WHITE,RED);
  Delay(5000);
  Lcd_Clear(BLACK);
  Lcd_FastRectangle(0,220,319,239,RED,1);

  i = 0;
  TimingDelay = 100;
  while(TimingDelay > 0) {
    Rectangle(i+20,i+10,i+50,i+30, SOLID_LINE, NORMAL_LINE, RED);
    i++;
  }
  sprintf(buffer, "Rectangle (100ms): %d", i);
  Lcd_Text(8,222,buffer,19+2,WHITE,RED);
  Delay(5000);
  // End of Rectangle Test


}

u8 Hour, Minutes, Seconds = 0;
  // MenuPage 0 = Hour (Tenth)
  // MenuPage 1 = Hour (Ones)
  // MenuPage 2 = Minutes (Tenth)
  // MenuPage 3 = Minutes (Ones)
  // MenuPage 4 = Seconds (Tenth)
  // MenuPage 5 = Seconds (Ones)
  // MenuPage 6 = Confirmation


void Check_ClockItems_Touch(u16 TouchX, u16 TouchY)
{
  u8 i;
  for (i = 0; i < 10; i++) {
    if (TouchX >= 10 && TouchX <= 275 && TouchY >= 20*i+30 && TouchY <= 20*i+30+20) {
      while (GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_0) == 0) { GlobalMenuInterrupt(); } // Loop while touch screen is touched

      if (CurrentSelected == i || SingleClickMenu)
      {
        CurrentSelected = i;
        DoAction_ClockMenu(); // Menu item was already selected, so execute!
        break;
      } else {
        CurrentSelected = i;
        break;
      }
    }
  }
}


void SetClock (void)
{
  BreakNow = 0;

  CurrentSelected = 0;
  OldSelected = 0;

  MenuPage = 0;
  ChangeMenuTitle("Hour (Tenth)", 12);
  ChangeMenuItem(0, "0x", 2);
  ChangeMenuItem(1, "1x", 2);
  ChangeMenuItem(2, "2x", 2);
  ChangeMenuItem(3, "", 0);
  ChangeMenuItem(4, "", 0);
  ChangeMenuItem(5, "", 0);
  ChangeMenuItem(6, "", 0);
  ChangeMenuItem(7, "", 0);
  ChangeMenuItem(8, "", 0);
  ChangeMenuItem(9, "", 0);
  DrawMenuItems();

while (!BreakNow)
 {
  GlobalMenuInterrupt();

  while ((GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_0) == 0) && !BreakNow)
    {
      GlobalMenuInterrupt();

       Touch_screenToDisplay();
       TouchX = displayPoint.x;
       TouchY = displayPoint.y;

       if (TouchableMenuItems) { Check_ClockItems_Touch(TouchX, TouchY); }

       if (ButtonPressed(0, TouchX, TouchY)) {
         DrawButton(0, BTN_PRESSED, "", 0);
         while (GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_0) == 0)
         {
           GlobalMenuInterrupt();
         }
         if (CurrentSelected > 0) {CurrentSelected--;} else { CurrentSelected = 9; }
         DrawButton(0, 0, "", 0);
       }

       if (ButtonPressed(1, TouchX, TouchY)) {
         DrawButton(1, BTN_PRESSED, "", 0);
         while (GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_0) == 0)
         {
           GlobalMenuInterrupt();
         }
         if (CurrentSelected < 9) {CurrentSelected++;} else { CurrentSelected = 0; }
         DrawButton(1, 0, "", 0);
       }

       if (ButtonPressed(2, TouchX, TouchY)) {
         DrawButton(2, BTN_PRESSED, "OK", 2);
         while (GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_0) == 0)
         {
           GlobalMenuInterrupt();
         }
         DrawButton(2, 0, "OK", 2);  // Draw the normal (un-pressed) button
         DoAction_ClockMenu();
       }
    }
  }
}

void DoAction_ClockMenu(void)
{
         switch (MenuPage) {
         case 0:  // MenuPage 0 = Hour (Tenth)
           if (CurrentSelected > 2) { break; }
           Hour = CurrentSelected*10;
           MenuPage = 1;
           CurrentSelected = 0;
           OldSelected = 0;
           ChangeMenuTitle("Hour (Ones)", 11);
           ChangeMenuItem(0, "x0", 2);
           ChangeMenuItem(1, "x1", 2);
           ChangeMenuItem(2, "x2", 2);
           ChangeMenuItem(3, "x3", 2);
           ChangeMenuItem(4, "x4", 2);
           ChangeMenuItem(5, "x5", 2);
           ChangeMenuItem(6, "x6", 2);
           ChangeMenuItem(7, "x7", 2);
           ChangeMenuItem(8, "x8", 2);
           ChangeMenuItem(9, "x9", 2);
           DrawMenuItems();
           break;

         case 1:  // MenuPage 1 = Hour (Ones)
           Hour = Hour + CurrentSelected;
           if (Hour > 24) { // Hour not valid
             MenuPage = 0;
             CurrentSelected = 0;
             OldSelected = 0;
             ChangeMenuTitle("Hour (Tenth)", 12);
             ChangeMenuItem(0, "0x", 2);
             ChangeMenuItem(1, "1x", 2);
             ChangeMenuItem(2, "2x", 2);
             ChangeMenuItem(3, "", 0);
             ChangeMenuItem(4, "", 0);
             ChangeMenuItem(5, "", 0);
             ChangeMenuItem(6, "", 0);
             ChangeMenuItem(7, "", 0);
             ChangeMenuItem(8, "", 0);
             ChangeMenuItem(9, "", 0);
             DrawMenuItems();
             break;
           }
           MenuPage = 2;
           CurrentSelected = 0;
           OldSelected = 0;
           ChangeMenuTitle("Minutes (Tenth)", 15);
           ChangeMenuItem(0, "0x", 2);
           ChangeMenuItem(1, "1x", 2);
           ChangeMenuItem(2, "2x", 2);
           ChangeMenuItem(3, "3x", 2);
           ChangeMenuItem(4, "4x", 2);
           ChangeMenuItem(5, "5x", 2);
           ChangeMenuItem(6, "", 0);
           ChangeMenuItem(7, "", 0);
           ChangeMenuItem(8, "", 0);
           ChangeMenuItem(9, "", 0);
           DrawMenuItems();
           break;

         case 2:  // MenuPage 2 = Minutes (Tenth)
           if (CurrentSelected > 5) { break; }
           Minutes = CurrentSelected*10;
           MenuPage = 3;
           CurrentSelected = 0;
           OldSelected = 0;
           ChangeMenuTitle("Minutes (Ones)", 14);
           ChangeMenuItem(0, "x0", 2);
           ChangeMenuItem(1, "x1", 2);
           ChangeMenuItem(2, "x2", 2);
           ChangeMenuItem(3, "x3", 2);
           ChangeMenuItem(4, "x4", 2);
           ChangeMenuItem(5, "x5", 2);
           ChangeMenuItem(6, "x6", 2);
           ChangeMenuItem(7, "x7", 2);
           ChangeMenuItem(8, "x8", 2);
           ChangeMenuItem(9, "x9", 2);
           DrawMenuItems();
           break;

         case 3:  // MenuPage 3 = Minutes (Ones)
           Minutes = Minutes + CurrentSelected;
           MenuPage = 4;
           CurrentSelected = 0;
           OldSelected = 0;
           ChangeMenuTitle("Seconds (Tenth)", 15);
           ChangeMenuItem(0, "0x", 2);
           ChangeMenuItem(1, "1x", 2);
           ChangeMenuItem(2, "2x", 2);
           ChangeMenuItem(3, "3x", 2);
           ChangeMenuItem(4, "4x", 2);
           ChangeMenuItem(5, "5x", 2);
           ChangeMenuItem(6, "", 0);
           ChangeMenuItem(7, "", 0);
           ChangeMenuItem(8, "", 0);
           ChangeMenuItem(9, "", 0);
           DrawMenuItems();
           break;

         case 4:  // MenuPage 4 = Seconds (Tenth)
           if (CurrentSelected > 5) { break; }
           Seconds = CurrentSelected*10;
           MenuPage = 5;
           CurrentSelected = 0;
           OldSelected = 0;
           ChangeMenuTitle("Seconds (Ones)", 14);
           ChangeMenuItem(0, "x0", 2);
           ChangeMenuItem(1, "x1", 2);
           ChangeMenuItem(2, "x2", 2);
           ChangeMenuItem(3, "x3", 2);
           ChangeMenuItem(4, "x4", 2);
           ChangeMenuItem(5, "x5", 2);
           ChangeMenuItem(6, "x6", 2);
           ChangeMenuItem(7, "x7", 2);
           ChangeMenuItem(8, "x8", 2);
           ChangeMenuItem(9, "x9", 2);
           DrawMenuItems();
           break;

         case 5:  // MenuPage 5 = Seconds (Ones)
           Seconds = Seconds + CurrentSelected;
           MenuPage = 6;
           CurrentSelected = 3;
           OldSelected = 3;
           ChangeMenuTitle("Set Clock", 9);
           sprintf(buffer, "%0.2d:%0.2d:%0.2d", Hour, Minutes, Seconds);
           ChangeMenuItem(0, "So the time is:", 15);
           ChangeMenuItem(1, buffer, 8);
           ChangeMenuItem(2, "", 0);
           ChangeMenuItem(3, "Yes", 3);
           ChangeMenuItem(4, "No", 2);
           ChangeMenuItem(5, "", 0);
           ChangeMenuItem(6, "", 0);
           ChangeMenuItem(7, "", 0);
           ChangeMenuItem(8, "", 0);
           ChangeMenuItem(9, "", 0);
           DrawMenuItems();
           break;

         case 6:  // MenuPage 6 = Confirmation
           if (CurrentSelected == 3)
           {
             Lcd_Text(10+4, 20*6+30+3, "Setting Clock...", 16, WHITE, BLACK);
             Time_Adjust((Hour*3600 + Minutes*60 + Seconds));
             CurrentSelected = 6;
             OldSelected = 6;
             BreakNow = 1;
             break;
           }

           if (CurrentSelected == 4) {
             CurrentSelected = 6;
             OldSelected = 6;
             BreakNow = 1;
             break;
           }
           break;
         }
}

bool Old_SingleClickMenu;

void Check_SettingsItems_Touch(u16 TouchX, u16 TouchY)
{
  u8 i;
  for (i = 0; i < 10; i++) {
    if (TouchX >= 10 && TouchX <= 275 && TouchY >= 20*i+30 && TouchY <= 20*i+30+20) {
      while (GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_0) == 0) { GlobalMenuInterrupt(); } // Loop while touch screen is touched

      if (CurrentSelected == i || SingleClickMenu)
      {
        CurrentSelected = i;
        DoAction_SettingsMenu(); // Menu item was already selected, so execute!
        break;
      } else {
        CurrentSelected = i;
        break;
      }
    }
  }
}


void Settings (void)
{
  BreakNow = 0;

  CurrentSelected = 3;
  OldSelected = 3;

  Old_TouchableMenuItems = TouchableMenuItems;
  Old_SingleClickMenu = SingleClickMenu;

  MenuPage = 0;
  ChangeMenuTitle("Settings", 8);
  sprintf(buffer, "Touchable Menu Items: %0.1d", TouchableMenuItems);
  ChangeMenuItem(0, buffer, 23);
  sprintf(buffer, "Single Click Menu: %0.1d", SingleClickMenu);
  ChangeMenuItem(1, buffer, 20);
  ChangeMenuItem(2, "", 0);
  ChangeMenuItem(3, "Back", 4);
  ChangeMenuItem(4, "", 0);
  ChangeMenuItem(5, "", 0);
  ChangeMenuItem(6, "", 0);
  ChangeMenuItem(7, "", 0);
  ChangeMenuItem(8, "", 0);
  ChangeMenuItem(9, "", 0);
  DrawMenuItems();

while (!BreakNow)
 {
  GlobalMenuInterrupt();

  while ((GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_0) == 0) && !BreakNow)
    {
      GlobalMenuInterrupt();

       Touch_screenToDisplay();
       TouchX = displayPoint.x;
       TouchY = displayPoint.y;

       if (TouchableMenuItems) { Check_SettingsItems_Touch(TouchX, TouchY); }

       if (ButtonPressed(0, TouchX, TouchY)) {
         DrawButton(0, BTN_PRESSED, "", 0);
         while (GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_0) == 0)
         {
           GlobalMenuInterrupt();
         }
         if (CurrentSelected > 0) {CurrentSelected--;} else { CurrentSelected = 9; }
         DrawButton(0, 0, "", 0);
       }

       if (ButtonPressed(1, TouchX, TouchY)) {
         DrawButton(1, BTN_PRESSED, "", 0);
         while (GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_0) == 0)
         {
           GlobalMenuInterrupt();
         }
         if (CurrentSelected < 9) {CurrentSelected++;} else { CurrentSelected = 0; }
         DrawButton(1, 0, "", 0);
       }

       if (ButtonPressed(2, TouchX, TouchY)) {
         DrawButton(2, BTN_PRESSED, "OK", 2);
         while (GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_0) == 0)
         {
           GlobalMenuInterrupt();
         }
         DrawButton(2, 0, "OK", 2);  // Draw the normal (un-pressed) button
         DoAction_SettingsMenu();
       }
    }
  }
}

void DoAction_SettingsMenu(void)
{
         switch (MenuPage) {
         case 0:  // MenuPage 6 = Confirmation
           if (CurrentSelected == 0)
           {
             TouchableMenuItems = !TouchableMenuItems;
             sprintf(buffer, "Touchable Menu Items: %0.1d", TouchableMenuItems);
             ChangeMenuItem(0, buffer, 23);
             DrawMenuItems();
           }

           if (CurrentSelected == 1)
           {
             int test;
             SingleClickMenu = !SingleClickMenu;
             sprintf(buffer, "Single Click Menu: %0.1d", SingleClickMenu);
             ChangeMenuItem(1, buffer, 20);
             DrawMenuItems();
           }

           if (CurrentSelected == 3) {
             CurrentSelected = 7;
             OldSelected = 7;
             if (Old_TouchableMenuItems != TouchableMenuItems || Old_SingleClickMenu != SingleClickMenu)
             {
               Lcd_Text(10+4, 20*6+30+3, "Saving Settings...", 18, WHITE, BLACK);
               SaveSettingsToFlash();
             }
             BreakNow = 1;
             break;
           }
         }
}


void Check_AboutItems_Touch(u16 TouchX, u16 TouchY)
{
  u8 i;
  for (i = 0; i < 10; i++) {
    if (TouchX >= 10 && TouchX <= 275 && TouchY >= 20*i+30 && TouchY <= 20*i+30+20) {
      while (GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_0) == 0) { GlobalMenuInterrupt(); } // Loop while touch screen is touched

      if (CurrentSelected == i || SingleClickMenu)
      {
        CurrentSelected = i;
        DoAction_AboutMenu(); // Menu item was already selected, so execute!
        break;
      } else {
        CurrentSelected = i;
        break;
      }
    }
  }
}


void About (void)
{
  BreakNow = 0;

  CurrentSelected = 9;
  OldSelected = 9;

  MenuPage = 0;
  ChangeMenuTitle("About", 5);
  ChangeMenuItem(0, "Mini Computer - TKJ Electronics", 31);
  ChangeMenuItem(1, "Made by Thomas Jespersen", 24);
  ChangeMenuItem(2, "Marts 2010", 10);
  ChangeMenuItem(3, "", 0);
  sprintf(buffer, "Software Version: %0.2f", Version);
  ChangeMenuItem(4, buffer, 22);
  sprintf(buffer, "Serial: %0.2X%0.2X%0.2X%0.2X%0.2X%0.2X%0.2X%0.2X", Serial[0],Serial[1],Serial[2],Serial[3],Serial[4],Serial[5],Serial[6],Serial[7]);
  ChangeMenuItem(5, buffer, 24);
  ChangeMenuItem(6, "", 0);
  ChangeMenuItem(7, "", 0);
  ChangeMenuItem(8, "Specifications", 14);
  ChangeMenuItem(9, "Back", 4);
  DrawMenuItems();

while (!BreakNow)
 {
  GlobalMenuInterrupt();

  while ((GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_0) == 0) && !BreakNow)
    {
      GlobalMenuInterrupt();

       Touch_screenToDisplay();
       TouchX = displayPoint.x;
       TouchY = displayPoint.y;

       if (TouchableMenuItems) { Check_AboutItems_Touch(TouchX, TouchY); }

       if (ButtonPressed(0, TouchX, TouchY)) {
         DrawButton(0, BTN_PRESSED, "", 0);
         while (GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_0) == 0)
         {
           GlobalMenuInterrupt();
         }
         if (CurrentSelected > 0) {CurrentSelected--;} else { CurrentSelected = 9; }
         DrawButton(0, 0, "", 0);
       }

       if (ButtonPressed(1, TouchX, TouchY)) {
         DrawButton(1, BTN_PRESSED, "", 0);
         while (GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_0) == 0)
         {
           GlobalMenuInterrupt();
         }
         if (CurrentSelected < 9) {CurrentSelected++;} else { CurrentSelected = 0; }
         DrawButton(1, 0, "", 0);
       }

       if (ButtonPressed(2, TouchX, TouchY)) {
         DrawButton(2, BTN_PRESSED, "OK", 2);
         while (GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_0) == 0)
         {
           GlobalMenuInterrupt();
         }
         DrawButton(2, 0, "OK", 2);  // Draw the normal (un-pressed) button
         DoAction_AboutMenu();
       }
    }
  }
}

void DoAction_AboutMenu(void)
{
         switch (MenuPage) {
         case 0:
           if (CurrentSelected == 8)
           {
             CurrentSelected = 9;
             OldSelected = 9;

             MenuPage = 1;
             ChangeMenuTitle("About -> Specifications", 23);
             ChangeMenuItem(0, "Processor: STM32F103RET6", 24);
             ChangeMenuItem(1, "Clock Frequency: 72MHz (8MHz)", 29);
             ChangeMenuItem(2, "", 0);
             ChangeMenuItem(3, "", 0);
             ChangeMenuItem(4, "Factory Date:", 13);
             sprintf(buffer, "%0.2X/%0.2X %0.2X%0.2X - %0.2X:%0.2X:%0.2X GMT+1", FactoryDate[0],FactoryDate[1],FactoryDate[2],FactoryDate[3],FactoryDate[4],FactoryDate[5],FactoryDate[6]);
             ChangeMenuItem(5, buffer, 27);
             ChangeMenuItem(6, "", 0);
             ChangeMenuItem(7, "", 0);
             ChangeMenuItem(8, "", 0);
             ChangeMenuItem(9, "Back", 4);
             DrawMenuItems();
             break;
           }

           if (CurrentSelected == 9)
           {
             CurrentSelected = 0;
             OldSelected = 0;
             BreakNow = 1;
             break;
           }


         case 1:
           if (CurrentSelected == 9)
           {
             CurrentSelected = 9;
             OldSelected = 9;

             MenuPage = 0;
             ChangeMenuTitle("About", 5);
             ChangeMenuItem(0, "Mini Computer - TKJ Electronics", 31);
             ChangeMenuItem(1, "Made by Thomas Jespersen", 24);
             ChangeMenuItem(2, "Marts 2010", 10);
             ChangeMenuItem(3, "", 0);
             sprintf(buffer, "Software Version: %0.2f", Version);
             ChangeMenuItem(4, buffer, 22);
             sprintf(buffer, "Serial: %0.2X%0.2X%0.2X%0.2X%0.2X%0.2X%0.2X%0.2X", Serial[0],Serial[1],Serial[2],Serial[3],Serial[4],Serial[5],Serial[6],Serial[7]);
             ChangeMenuItem(5, buffer, 24);
             ChangeMenuItem(6, "", 0);
             ChangeMenuItem(7, "", 0);
             ChangeMenuItem(8, "Specifications", 14);
             ChangeMenuItem(9, "Back", 4);
             DrawMenuItems();
             break;
           }
         }
}




bool SDCardInitialized;
u8 SDitemCount; // Items count in root (files and folders)
u8 SDcontentPages; // Count of Pages needed to show SD content
u8 i;


void Check_SDCardItems_Touch(u16 TouchX, u16 TouchY)
{
  u8 i;
  for (i = 0; i < 10; i++) {
    if (TouchX >= 10 && TouchX <= 275 && TouchY >= 20*i+30 && TouchY <= 20*i+30+20) {
      while (GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_0) == 0) { GlobalMenuInterrupt(); } // Loop while touch screen is touched

      if (CurrentSelected == i || SingleClickMenu)
      {
        CurrentSelected = i;
        DoAction_SDCardMenu(); // Menu item was already selected, so execute!
        break;
      } else {
        CurrentSelected = i;
        break;
      }
    }
  }
}

void ReDraw_SDMenu(void)
{
  Lcd_Clear(BLACK);
  Lcd_FastRectangle(0,0,319,20,RED,1);
  DrawMenuTitle();

  Rectangle(3, 25, 275, 235, DASHED_LINE, THICK_LINE, WHITE);

  DrawMenuItems();

  Lcd_FastRectangle(280, 20, 319, 239, RED, 1);
  CreateButton(0, 285, 30, 310, 50, 10, 0, "", 0); // Up button
  CreateButton(1, 285, 200, 310, 220, 10, 0, "", 0); // Down button
  CreateButton(2, 285, 110, 310, 133, 0, 0, "OK", 2); // OK button
}


void SDCard (void)
{
  BreakNow = 0;

  CurrentSelected = 9;
  OldSelected = 9;

  MenuPage = 0;
  ChangeMenuTitle("SD Card", 7);
  ChangeMenuItem(0, "Initializing SD Card ", 20);
  ChangeMenuItem(1, "", 0);
  ChangeMenuItem(2, "", 0);
  ChangeMenuItem(3, "", 0);
  ChangeMenuItem(4, "", 0);
  ChangeMenuItem(5, "", 0);
  ChangeMenuItem(6, "", 0);
  ChangeMenuItem(7, "", 0);
  ChangeMenuItem(8, "", 0);
  ChangeMenuItem(9, "Back", 4);
  DrawMenuItems();

  /* Initialize efsl */
  if(efs_init(&efs,0)!=0){
    ChangeMenuItem(1, "Error Initializing SD Card", 26);
    SDCardInitialized = 0;
  } else {

    ls_openDir(&list, &efs.myFs, "/img/");

    i = 0;
    SDitemCount = 0; // Items count in root (files and folders)
    SDcontentPages = 1; // Count of Pages needed to show SD content
    while (ls_getNext(&list) == 0)
    {
      i++;
      SDitemCount++;
      if (i == 9) {
        SDcontentPages++;
        i = 1;
      }
    }
    ChangeMenuItem(1, "SD Card Initialized", 19);
    ChangeMenuItem(3, "Show content of SD card", 23);

    SDCardInitialized = 1;
  }
  TP_Init(); // Remember to initialize Touch Panel again after communicating with the SD card
  DrawMenuItems();

while (!BreakNow)
 {
  GlobalMenuInterrupt();

  while ((GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_0) == 0) && !BreakNow)
    {
      GlobalMenuInterrupt();

       Touch_screenToDisplay();
       TouchX = displayPoint.x;
       TouchY = displayPoint.y;

       if (TouchableMenuItems) { Check_SDCardItems_Touch(TouchX, TouchY); }

       if (ButtonPressed(0, TouchX, TouchY)) {
         DrawButton(0, BTN_PRESSED, "", 0);
         while (GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_0) == 0)
         {
           GlobalMenuInterrupt();
         }
         if (CurrentSelected > 0) {CurrentSelected--;} else { CurrentSelected = 9; }
         DrawButton(0, 0, "", 0);
       }

       if (ButtonPressed(1, TouchX, TouchY)) {
         DrawButton(1, BTN_PRESSED, "", 0);
         while (GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_0) == 0)
         {
           GlobalMenuInterrupt();
         }
         if (CurrentSelected < 9) {CurrentSelected++;} else { CurrentSelected = 0; }
         DrawButton(1, 0, "", 0);
       }

       if (ButtonPressed(2, TouchX, TouchY)) {
         DrawButton(2, BTN_PRESSED, "OK", 2);
         while (GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_0) == 0)
         {
           GlobalMenuInterrupt();
         }
         DrawButton(2, 0, "OK", 2);  // Draw the normal (un-pressed) button
         DoAction_SDCardMenu();
       }
    }
  }
}

void DoAction_SDCardMenu(void)
{
         switch (MenuPage) {
         case 0:
           if (CurrentSelected == 3 && SDCardInitialized)
           {
             if (SDcontentPages > 1) {
               CurrentSelected = 8; // Highlight Next
               OldSelected = 8;
             } else {
               CurrentSelected = 9; // Highlight Back, because Next isn't there
               OldSelected = 9;
             }

             MenuPage = 1;
             sprintf(buffer,"Page 1/%i", SDcontentPages);
             ChangeMenuTitle(buffer, 8);
             PrintFilesFrom(0);

             if (SDcontentPages > 1) {
               ChangeMenuItem(8, "Next", 4);
             }
             ChangeMenuItem(9, "Back", 4);


             DrawMenuItems();

             break;
           }

           if (CurrentSelected == 9)
           {
             if (SDCardInitialized)
             {
               /* Close filesystem */
               fs_umount(&efs.myFs);
             }

             CurrentSelected = 0;
             OldSelected = 0;
             BreakNow = 1;
             break;
           }

         default:
           if (CurrentSelected < 8)
           {
             if (menuItems_length[CurrentSelected] >= 10 && menuItems[CurrentSelected][8] == 'B' && menuItems[CurrentSelected][9] == 'I' && menuItems[CurrentSelected][10] == 'N')  // The file type is BIN
               {
                 ShowFile(CurrentSelected);
                 ReDraw_SDMenu();
               }
             if (menuItems_length[CurrentSelected] >= 10 && menuItems[CurrentSelected][8] == 'M' && menuItems[CurrentSelected][9] == 'I' && menuItems[CurrentSelected][10] == 'D')  // The file type is MID (Midi)
               {
                 PlaySound(CurrentSelected);
                 ReDraw_SDMenu();
               }
             if (menuItems_length[CurrentSelected] >= 10 && menuItems[CurrentSelected][8] == 'M' && menuItems[CurrentSelected][9] == 'P' && menuItems[CurrentSelected][10] == '3')  // The file type is MP3
               {
                 PlaySound(CurrentSelected);
                 ReDraw_SDMenu();
               }
             if (menuItems_length[CurrentSelected] >= 10 && menuItems[CurrentSelected][8] == 'W' && menuItems[CurrentSelected][9] == 'A' && menuItems[CurrentSelected][10] == 'V')  // The file type is WAV
               {
                 PlaySound(CurrentSelected);
                 ReDraw_SDMenu();
               }
             // Code here for pressing on file or folder
             break;
           }

           if (CurrentSelected == 8 && MenuPage+1 <= SDcontentPages)
           {
             MenuPage = MenuPage+1;
             sprintf(buffer,"Page %i/%i", MenuPage, SDcontentPages);
             ChangeMenuTitle(buffer, 8);
             PrintFilesFrom((MenuPage-1)*7);

             if (MenuPage+1 <= SDcontentPages) {
               ChangeMenuItem(8, "Next", 4);
             } else {
               CurrentSelected = 9;
               OldSelected = 9;
             }
             ChangeMenuItem(9, "Back", 4);

             DrawMenuItems();
             break;
           }

           if (CurrentSelected == 9 && MenuPage > 1)
           {
             MenuPage = MenuPage-1;
             sprintf(buffer,"Page %i/%i", MenuPage, SDcontentPages);
             ChangeMenuTitle(buffer, 8);
             PrintFilesFrom((MenuPage-1)*7);

             if (MenuPage+1 <= SDcontentPages) {
               ChangeMenuItem(8, "Next", 4);
             } else {
               CurrentSelected = 9;
               OldSelected = 9;
             }
             ChangeMenuItem(9, "Back", 4);

             DrawMenuItems();
             break;
           }

           if (CurrentSelected == 9 && MenuPage == 1)
           {
             MenuPage = 0;
             ChangeMenuTitle("SD Card", 7);
             ChangeMenuItem(0, "Initializing SD Card", 20);
             ChangeMenuItem(2, "", 0);
             ChangeMenuItem(3, "", 0);
             if(!SDCardInitialized){
               ChangeMenuItem(1, "Error Initializing SD Card", 26);
               SDCardInitialized = 0;
             } else {
               ChangeMenuItem(1, "SD Card Initialized", 19);
               ChangeMenuItem(3, "Show content of SD card", 23);
               SDCardInitialized = 1;
             }
             ChangeMenuItem(4, "", 0);
             ChangeMenuItem(5, "", 0);
             ChangeMenuItem(6, "", 0);
             ChangeMenuItem(7, "", 0);
             ChangeMenuItem(8, "", 0);
             ChangeMenuItem(9, "Back", 4);
             DrawMenuItems();
             break;
           }
         }
}

void PrintFilesFrom(u8 startFrom)
{
  ChangeMenuItem(0, "", 0);
  ChangeMenuItem(1, "", 0);
  ChangeMenuItem(2, "", 0);
  ChangeMenuItem(3, "", 0);
  ChangeMenuItem(4, "", 0);
  ChangeMenuItem(5, "", 0);
  ChangeMenuItem(6, "", 0);
  ChangeMenuItem(7, "", 0);
  ChangeMenuItem(8, "", 0);
  ChangeMenuItem(9, "", 0);

  if(efs_init(&efs,0)!=0){
    ChangeMenuTitle("SD Card - Error", 15);
    ChangeMenuItem(0, "SD Card was removed!", 20);
    ChangeMenuItem(1, "Returning to main menu", 22);
    DrawMenuItems();

    Delay(1000);

    fs_umount(&efs.myFs);
    TP_Init(); // Remember to initialize Touch Panel again after communicating with the SD card
    CurrentSelected = 0;
    OldSelected = 0;
    BreakNow = 1;
  } else {

    // Open the directory
    ls_openDir(&list, &efs.myFs, "/img/");
    if (ls_getNext(&list) == 0) { // There are files to show
      if (startFrom > 0) {
        for (i = 0; i < startFrom; i++) {
          ls_getNext(&list); // Get next item until we come to startFrom
        }
      }
      for (i = 0; i <= 7; i++) {
        ChangeMenuItem(i, list.currentEntry.FileName, LIST_MAXLENFILENAME-1);
        if (ls_getNext(&list) != 0) { break; }
      }
    }

    TP_Init(); // Remember to initialize Touch Panel again after communicating with the SD card
  }
}

void ShowFile(u8 currentItem)
{
  bool ReturnToList = 0;
  unsigned char filename[16];
  if(efs_init(&efs,0)!=0){
    ChangeMenuTitle("SD Card - Error", 15);
    ChangeMenuItem(0, "SD Card was removed!", 20);
    ChangeMenuItem(1, "Returning to main menu", 22);
    DrawMenuItems();

    Delay(1000);

    fs_umount(&efs.myFs);
    TP_Init(); // Remember to initialize Touch Panel again after communicating with the SD card
    CurrentSelected = 0;
    OldSelected = 0;
    BreakNow = 1;
  } else {

    u8 i2 = 4;
    for (i = 0; i < LIST_MAXLENFILENAME-3; i++)
    {
      if (menuItems[currentItem][i] != 32) {
        filename[i2] = menuItems[currentItem][i];
        i2++;
      }
    }
    filename[0] = 'i';
    filename[1] = 'm';
    filename[2] = 'g';
    filename[3] = '/';
    filename[i2-1] = 46; // Add a dot
    filename[i2] = menuItems[currentItem][i-1];
    filename[i2+1] = menuItems[currentItem][i];
    filename[i2+2] = menuItems[currentItem][i+1];

    Lcd_Clear(BLACK);
    Lcd_Text(200,220,filename, 15, WHITE, BLACK);
    if (DispPic_SD_Fast(0,0,filename) != 0) {
      //Lcd_Text(0,10,"Error loading image",19,BLACK,WHITE);
      ReturnToList = 1;
    }
  }

  TP_Init(); // Remember to initialize Touch Panel again after communicating with the SD card

  while (!ReturnToList) {
    if (GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_0) == 0)
    {
      while (GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_0) == 0) { }
      ReturnToList = 1;
      break;// Exit the image show and redraw the SD card menu
    }
  }
}

void PlaySound(u8 currentItem)
{
  bool ReturnToList = 0;
  unsigned char filename[16];
  if(efs_init(&efs,0)!=0){
    ChangeMenuTitle("SD Card - Error", 15);
    ChangeMenuItem(0, "SD Card was removed!", 20);
    ChangeMenuItem(1, "Returning to main menu", 22);
    DrawMenuItems();

    Delay(1000);

    fs_umount(&efs.myFs);
    TP_Init(); // Remember to initialize Touch Panel again after communicating with the SD card
    CurrentSelected = 0;
    OldSelected = 0;
    BreakNow = 1;
  } else {

    u8 i2 = 4;
    for (i = 0; i < LIST_MAXLENFILENAME-3; i++)
    {
      if (menuItems[currentItem][i] != 32) {
        filename[i2] = menuItems[currentItem][i];
        i2++;
      }
    }
    filename[0] = 'i';
    filename[1] = 'm';
    filename[2] = 'g';
    filename[3] = '/';
    filename[i2-1] = 46; // Add a dot
    filename[i2] = menuItems[currentItem][i-1];
    filename[i2+1] = menuItems[currentItem][i];
    filename[i2+2] = menuItems[currentItem][i+1];

    Lcd_Clear(BLACK);
    Lcd_Text(192,220,filename, 16, WHITE, BLACK);

    VS1053_Configuration();
    VS1053_Init(SM_SDISHARE | SM_SDINEW); // sets sci_mode register, SM_SDISHARE,
                                                                 // SM_SDINEW.  pg 25, 26
    VS1053_Write(VS_CLOCKF, 0xE000); // SC_MULT = 7, SC_ADD = 0
    VS1053_SetVolume(100);

    unsigned int e,i;
    u32 current_pos;
    current_pos = 0;
    unsigned char sound_buffer[500];

    // Open file for reading
    if (file_fopen(&file,&efs.myFs,filename,'r')!=0){
      return(-1); // Error
    } else {
      while((e=file_read(&file,500,sound_buffer))) {
        for (i = 0; i < e; i++) {
          VS1053_SendStreamByte(sound_buffer[i]);
          if (GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_0) == 0)
          {
            while (GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_0) == 0) { }
            ReturnToList = 1;
            break;// Exit the image show and redraw the SD card menu
          }
        }
        //current_pos = current_pos + e;
        //sprintf(buffer, "Playing: %7.0d", current_pos);
        //Lcd_Text(192,200, buffer, 16, WHITE, BLACK);

      }
      // Close file
      file_fclose(&file);
    }
  }

  TP_Init(); // Remember to initialize Touch Panel again after communicating with the SD card
}



/******************* (C) COPYRIGHT 2007 STMicroelectronics *****END OF FILE****/
